package com.vdocipher.flutter.vdoplayer;

import androidx.annotation.Nullable;

import com.vdocipher.aegis.media.ErrorDescription;
import com.vdocipher.aegis.media.MediaInfo;
import com.vdocipher.aegis.media.Track;
import com.vdocipher.aegis.offline.DownloadOptions;
import com.vdocipher.aegis.offline.DownloadRequest;
import com.vdocipher.aegis.offline.DownloadSelections;
import com.vdocipher.aegis.offline.DownloadStatus;
import com.vdocipher.aegis.player.VdoPlayer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class MessageUtils {
    public static HashMap<String, Object> loadResultMap(@Nullable MediaInfo mediaInfo) {
        HashMap<String, Object> loadResultMap = new HashMap<>();
        if (mediaInfo == null) {
            loadResultMap.put("didLoad", false);
        } else {
            loadResultMap.put("didLoad", true);
            loadResultMap.put("mediaInfo", mediaInfoMap(mediaInfo));
            loadResultMap.put("playWhenReady", true);
        }
        return loadResultMap;
    }

    public static HashMap<String, Object> mediaInfoMap(MediaInfo mediaInfo) {
        HashMap<String, Object> mediaInfoMap = new HashMap<>();
        mediaInfoMap.put("mediaId", mediaInfo.mediaId);
        mediaInfoMap.put("title", mediaInfo.title);
        mediaInfoMap.put("duration", mediaInfo.duration);
        mediaInfoMap.put("description", mediaInfo.description);
        return mediaInfoMap;
    }

    public static MediaInfo mediaInfoFromMap(HashMap<String, Object> mediaInfoMap) {
        String mediaId = (String) mediaInfoMap.get("mediaId");
        int type = MediaInfo.TYPE_INFO;
        String title = (String) mediaInfoMap.get("title");
        String description = (String) mediaInfoMap.get("description");
        int duration = (int) mediaInfoMap.get("duration");
        return new MediaInfo(type, mediaId, title, description, duration);
    }

    public static ArrayList<HashMap<String, Object>> trackMapList(Track[] tracks) {
        ArrayList<HashMap<String, Object>> trackList = new ArrayList<>();
        for (Track track : tracks) {
            trackList.add(trackMap(track));
        }
        return trackList;
    }

    public static HashMap<String, Object> trackMap(Track track) {
        HashMap<String, Object> trackMap = new HashMap<>();
        trackMap.put("id", track.id);
        switch (track.type) {
            case Track.TYPE_AUDIO:
                trackMap.put("type", "audio");
                trackMap.put("language", track.language);
                trackMap.put("bitrate", track.bitrate);
                break;
            case Track.TYPE_VIDEO:
                trackMap.put("type", "video");
                trackMap.put("bitrate", track.bitrate);
                trackMap.put("width", track.width);
                trackMap.put("height", track.height);
                break;
            case Track.TYPE_CAPTIONS:
                trackMap.put("type", "subtitle");
                trackMap.put("language", track.language);
                break;
            default:
                break;
        }
        return trackMap;
    }

    public static Track trackFromMap(HashMap<String, Object> trackMap) {
        switch ((String) trackMap.get("type")) {
            case "audio": {
                String language = (String) trackMap.get("language");
                int bitrate = (int) trackMap.get("bitrate");
                int id = (int) trackMap.get("id");
                return new Track(id, Track.TYPE_AUDIO, 0, 0, bitrate, language, null);
            }
            case "video": {
                int bitrate = (int) trackMap.get("bitrate");
                int height = (int) trackMap.get("height");
                int width = (int) trackMap.get("width");
                int id = (int) trackMap.get("id");
                return new Track(id, Track.TYPE_VIDEO, height, width, bitrate, null, null);
            }
        }
        return null;
    }

    public static HashMap<String, Object> errorInfoMap(ErrorDescription error) {
        HashMap<String, Object> errorInfoMap = new HashMap<>();
        errorInfoMap.put("errorCode", error.errorCode);
        errorInfoMap.put("errorMsg", error.errorMsg);
        errorInfoMap.put("httpStatusCode", error.httpStatusCode);
        return errorInfoMap;
    }

    public static HashMap<String, Object> downloadOptionsMap(DownloadOptions downloadOptions) {
        HashMap<String, Object> downloadOptionsMap = new HashMap<>();
        downloadOptionsMap.put("mediaId", downloadOptions.mediaId);
        downloadOptionsMap.put("mediaInfo", mediaInfoMap(downloadOptions.mediaInfo));
        downloadOptionsMap.put("availableTracks", trackMapList(downloadOptions.availableTracks));
        return downloadOptionsMap;
    }

    public static HashMap<String, Object> downloadStatusMap(DownloadStatus downloadStatus) {
        HashMap<String, Object> downloadStatusMap = new HashMap<>();
        downloadStatusMap.put("mediaInfo", mediaInfoMap(downloadStatus.mediaInfo));
        downloadStatusMap.put("status", downloadStatus.status);
        downloadStatusMap.put("reason", downloadStatus.reason);
        downloadStatusMap.put("reasonDescription", downloadStatus.reasonDescription);
        downloadStatusMap.put("percentDownloaded", downloadStatus.downloadPercent);
        downloadStatusMap.put("poster", downloadStatus.poster);
        return downloadStatusMap;
    }

    public static String playerStateName(int vdoPlayerState) {
        switch (vdoPlayerState) {
            case VdoPlayer.STATE_IDLE:
                return "idle";
            case VdoPlayer.STATE_BUFFERING:
                return "buffering";
            case VdoPlayer.STATE_READY:
                return "ready";
            case VdoPlayer.STATE_ENDED:
                return "ended";
            default:
                return "unknown";
        }
    }

    public static DownloadRequest downloadRequestFromMap(DownloadOptions downloadOptions, HashMap<String, Object> downloadRequestMap) {
        String safetyNetApiKey = null;
        if (downloadRequestMap.containsKey("safetyNetApiKey"))
            safetyNetApiKey = (String) downloadRequestMap.get("safetyNetApiKey");
        DownloadSelections downloadSelections = downloadSelectionsFromMap(downloadOptions, (HashMap<String, Object>) downloadRequestMap.get("downloadSelections"));
        return new DownloadRequest.Builder(downloadSelections)
                .setSafetyNetApiKey(safetyNetApiKey).build();
    }

    static DownloadSelections downloadSelectionsFromMap(DownloadOptions downloadOptions, HashMap<String, Object> downloadSelectionMap) {
        int selectedVideoTrackId = (int) downloadSelectionMap.get("selectedVideoTrackId");
        int selectedAudioTrackId = -1;
        if (downloadSelectionMap.containsKey("selectedAudioTrackId")) {
            selectedAudioTrackId = (int) downloadSelectionMap.get("selectedAudioTrackId");
        }
        int selectedVideoIndex = 0;
        int selectedAudioIndex = 0;
        int index = 0;
        for (Track track : downloadOptions.availableTracks) {
            if (track.id == selectedVideoTrackId)
                selectedVideoIndex = index;
            if (track.id == selectedAudioTrackId)
                selectedAudioIndex = index;
            index++;
        }
        if (selectedAudioTrackId != -1)
            return new DownloadSelections(downloadOptions, new int[]{selectedAudioIndex, selectedVideoIndex});
        else
            return new DownloadSelections(downloadOptions, new int[]{selectedVideoIndex});
    }

    static DownloadOptions downloadOptionsFromMap(HashMap<String, Object> downloadOptionsMap) {
        String mediaId = (String) downloadOptionsMap.get("mediaId");
        MediaInfo mediaInfo = MessageUtils.mediaInfoFromMap((HashMap<String, Object>) downloadOptionsMap.get("mediaInfo"));
        List<HashMap<String, Object>> videoTracks = (List<HashMap<String, Object>>) downloadOptionsMap.get("allVideo");
        List<HashMap<String, Object>> audioTracks = (List<HashMap<String, Object>>) downloadOptionsMap.get("allAudio");
        List<Track> tracks = new ArrayList<>();
        for (HashMap<String, Object> track : videoTracks) {
            tracks.add(MessageUtils.trackFromMap(track));
        }
        for (HashMap<String, Object> track : audioTracks) {
            tracks.add(MessageUtils.trackFromMap(track));
        }
        return new DownloadOptions(mediaId, mediaInfo, tracks.toArray(new Track[tracks.size()]));
    }
}
