package com.vdocipher.flutter.vdoplayer;

import android.app.Activity;
import android.content.Context;
import android.content.res.Configuration;
import android.media.AudioManager;
import android.os.Bundle;
import android.support.v4.media.MediaMetadataCompat;
import android.support.v4.media.session.MediaControllerCompat;
import android.support.v4.media.session.MediaSessionCompat;
import android.support.v4.media.session.PlaybackStateCompat;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.PictureInPictureModeChangedInfo;
import androidx.core.util.Consumer;
import androidx.fragment.app.FragmentActivity;
import androidx.fragment.app.FragmentManager;

import com.vdocipher.aegis.media.ErrorDescription;
import com.vdocipher.aegis.media.Track;
import com.vdocipher.aegis.player.PlayerHost;
import com.vdocipher.aegis.player.VdoInitParams;
import com.vdocipher.aegis.player.VdoPlayer;
import com.vdocipher.aegis.ui.view.VdoPlayerControlView;
import com.vdocipher.aegis.ui.view.VdoPlayerUIFragment;

import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.platform.PlatformView;

class VdoPlayerUiPlatformView implements PlatformView, MethodChannel.MethodCallHandler, PlayerHost.InitializationListener, VdoPlayer.PlaybackEventListener, Consumer<Configuration> {
    private static final String TAG = "VdoPlayerUiPlatformView";

    private static final int IMPLEMENTATION_ERROR = -101;

    @NonNull
    private final LinearLayout linearLayout;
    private MethodChannel methodChannel;
    @Nullable
    private EventChannel.EventSink playerEventSink;


    @Nullable
    private VdoPlayer vdoPlayer;
    private final Activity activity;
    private final int id;
    VdoPlayerUIFragment vdoPlayerUIFragment;
    VdoPlayerControlView vdoPlayerControlView;
    @Nullable
    private MethodChannel.Result loadResult;
    private final boolean showControls;
    final int frameLayoutId;
    //Some random offset to define framelayout id.
    private static int FRAME_LAYOUT_OFFSET = 30;
    private final WeakReference<AudioManager> audioManager;

    private MediaSessionCompat mSession;

    public static final long MEDIA_ACTIONS_PLAY_PAUSE =
            PlaybackStateCompat.ACTION_PLAY
                    | PlaybackStateCompat.ACTION_PAUSE
                    | PlaybackStateCompat.ACTION_PLAY_PAUSE;

    VdoPlayerUiPlatformView(@NonNull Context context, final int id, BinaryMessenger binaryMessenger, @Nullable final Map<String, Object> creationParams, Activity activity) {

        this.activity = activity;
        this.id = id;
        showControls = (creationParams != null && creationParams.containsKey("showControls")) ? (Boolean) creationParams.get("showControls") : false;
        linearLayout = new LinearLayout(context);
        linearLayout.setClickable(false);
        vdoPlayerUIFragment = new VdoPlayerUIFragment();
        Bundle bundle = new Bundle();
        bundle.putBoolean("showControls", showControls);
        vdoPlayerUIFragment.setArguments(bundle);
        ViewGroup.LayoutParams vParams = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        FrameLayout frameLayout = new FrameLayout(context);
        frameLayout.setLayoutParams(vParams);
        frameLayoutId = id  + FRAME_LAYOUT_OFFSET;
        frameLayout.setId(frameLayoutId);

        linearLayout.addView(frameLayout);

        frameLayout.addOnAttachStateChangeListener(new View.OnAttachStateChangeListener() {
            @Override
            public void onViewAttachedToWindow(View v) {
                initializePlayerFragment();
            }

            @Override
            public void onViewDetachedFromWindow(View v) {
            }
        });

        methodChannel = new MethodChannel(binaryMessenger, "com.vdocipher.flutter.vdoplayer_" + id);
        methodChannel.setMethodCallHandler(this);

        EventChannel eventChannel = new EventChannel(binaryMessenger, "com.vdocipher.flutter.vdoplayer.events_" + id);
        eventChannel.setStreamHandler(new EventChannel.StreamHandler() {
            @Override
            public void onListen(Object arguments, EventChannel.EventSink eventSink) {
                playerEventSink = eventSink;
            }

            @Override
            public void onCancel(Object arguments) {
                playerEventSink = null;
            }
        });
        AudioManager am = (AudioManager) activity.getBaseContext().getSystemService(Context.AUDIO_SERVICE);
        audioManager = new WeakReference<>(am);
    }

    private void initializePlayerFragment() {
        if (activity instanceof FragmentActivity) {
            FragmentManager fm = ((FragmentActivity) activity).getSupportFragmentManager();
            ((FragmentActivity) activity).addOnConfigurationChangedListener(this);
            fm.beginTransaction().replace(frameLayoutId, vdoPlayerUIFragment, VdoPlayerUIFragment.class.getName()).commitNow();
            vdoPlayerControlView = Objects.requireNonNull(vdoPlayerUIFragment).requireView().findViewById(R.id.vdo_player_control_view);
            checkCurrentOrientationNotify();
            vdoPlayerUIFragment.initialize(this);
            ((FragmentActivity) activity).addOnPictureInPictureModeChangedListener(pictureModeChangedInfoConsumer);
        } else {
            String fullClassName;
            try {
                fullClassName = Class.forName("io.flutter.embedding.android.FlutterFragmentActivity").getSimpleName();
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
                fullClassName = "FlutterFragmentActivity";
            }
            String errorMsg = activity.getClass().getSimpleName() + " is not a subclass of " + fullClassName + ", try extending from " + fullClassName + " instead";
            Log.e(TAG, errorMsg);
            ErrorDescription errorDescription = new ErrorDescription(IMPLEMENTATION_ERROR, errorMsg, -1);
            Map<String, Object> arguments = new HashMap<>();
            arguments.put("errorDescription", MessageUtils.errorInfoMap(errorDescription));
            methodChannel.invokeMethod("onPlatformException", arguments);
        }
    }

    @Override
    public void onInitializationSuccess(PlayerHost playerHost, VdoPlayer player, boolean wasRestored) {
        this.vdoPlayer = player;
        vdoPlayer.addPlaybackEventListener(this);
        methodChannel.invokeMethod("onPlatformPlayerCreated", id);
        if (!showControls)
            initializeMediaSession();
    }

    @Override
    public void onInitializationFailure(PlayerHost playerHost, ErrorDescription errorDescription) {

    }

    Consumer<PictureInPictureModeChangedInfo> pictureModeChangedInfoConsumer = pictureInPictureModeChangedInfo ->
    {
        if(methodChannel!=null) {
            methodChannel.invokeMethod("onPictureInPictureModeChanged", pictureInPictureModeChangedInfo.isInPictureInPictureMode());
        }
    };

    private void initializeMediaSession() {
        mSession = new MediaSessionCompat(activity, TAG);
        mSession.setActive(true);
        MediaControllerCompat.setMediaController(activity, mSession.getController());

        MediaMetadataCompat metadata = new MediaMetadataCompat.Builder().build();
        mSession.setMetadata(metadata);

        MediaSessionCallback mMediaSessionCallback = new MediaSessionCallback(vdoPlayer);
        mSession.setCallback(mMediaSessionCallback);

        boolean playing = vdoPlayer != null && vdoPlayer.getPlaybackState() != VdoPlayer.STATE_IDLE && vdoPlayer.getPlaybackState() != VdoPlayer.STATE_ENDED && vdoPlayer.getPlayWhenReady();

        int state = playing ? PlaybackStateCompat.STATE_PLAYING : PlaybackStateCompat.STATE_PAUSED;

        if (vdoPlayer != null && !showControls) {
            updatePlaybackState(state, MEDIA_ACTIONS_PLAY_PAUSE, vdoPlayer.getCurrentTime(), vdoPlayer.getPlaybackSpeed());
        }
    }

    /**
     * Overloaded method that persists previously set media actions.
     *
     * @param state    The state of the video, e.g. playing, paused, etc.
     * @param position The position of playback in the video.
     */
    private void updatePlaybackState(
            @PlaybackStateCompat.State int state, long position, float playBackSpeed) {
        long actions = mSession.getController().getPlaybackState().getActions();
        updatePlaybackState(state, actions, position, playBackSpeed);
    }

    private void updatePlaybackState(
            @PlaybackStateCompat.State int state, long playbackActions, long position, float playBackSpeed) {
        PlaybackStateCompat.Builder builder =
                new PlaybackStateCompat.Builder()
                        .setActions(playbackActions)
                        .setState(state, position, playBackSpeed);
        mSession.setPlaybackState(builder.build());
    }

    void checkCurrentOrientationNotify() {
        if (vdoPlayerUIFragment != null) {
            boolean isFullScreen = vdoPlayerUIFragment.isFullScreen();
            methodChannel.invokeMethod("onFullscreenAction", isFullScreen);
        }
    }


    @NonNull
    @Override
    public View getView() {
        return linearLayout;
    }

    @Override
    public void dispose() {
        if (vdoPlayerUIFragment != null) {
            //Removing fragment from backstack to release player and clear memory.
            if (vdoPlayer != null) {
                vdoPlayer.stop();
            }
            FragmentManager fragmentManager = ((FragmentActivity) activity).getSupportFragmentManager();
            if (!fragmentManager.isDestroyed()) {
                fragmentManager.beginTransaction().remove(Objects.requireNonNull(fragmentManager.findFragmentById(vdoPlayerUIFragment.getId()))).commitAllowingStateLoss();
            }
            vdoPlayerUIFragment = null;
        }

        if (audioManager != null && !showControls) {
            audioManager.get().abandonAudioFocus(audioFocusChangeListener);
        }
        if(mSession!=null && !showControls){
            mSession.release();
        }
        if (activity != null) {
            ((FragmentActivity) activity).removeOnPictureInPictureModeChangedListener(pictureModeChangedInfoConsumer);
        }
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        if (vdoPlayer == null) {
            return;
        }

        switch (call.method) {
            case "load":
                // TODO handle load args parse failure
                Map<String, Object> loadArgs = (Map<String, Object>) call.arguments;

                final boolean restore = (boolean) loadArgs.get("restore");
                if (!restore) {
                    // Load new provided EmbedInfo
                    Map<String, Object> embedInfo = (Map<String, Object>) loadArgs.get("embedInfo");
                    VdoInitParams initParams = new EmbedInfoToVdoInitParams(embedInfo).convert();
                    HashMap<String, Object> configMap = new HashMap<>();
                    initParams.configMap = configMap;
                    vdoPlayer.load(initParams);
                }
                loadResult = result;
                break;
            case "setPlayWhenReady":
                vdoPlayer.setPlayWhenReady((boolean) call.arguments);
                break;

            case "getPlayWhenReady":
                result.success(vdoPlayer.getPlayWhenReady());
                break;

            case "getPlaybackAndBufferedPositions":
                HashMap<String, Object> positions = new HashMap<>();
                long currentTime = vdoPlayer.getCurrentTime();
                long bufferedTime = vdoPlayer.getBufferedTime();
                positions.put("position", currentTime >= 0 ? currentTime : 0);
                positions.put("bufferedPosition", bufferedTime >= 0 ? bufferedTime : 0);
                result.success(positions);
                break;

            case "getDuration":
                result.success(vdoPlayer.getDuration());
                break;

            case "seek":
                vdoPlayer.seekTo((int) call.arguments);
                result.success(null);
                break;
            case "stop":
                result.success(null);
                break;
            case "packUp":
                result.success(null);
                break;
            case "setPlaybackSpeed":
                double speed = (double) call.arguments;
                vdoPlayer.setPlaybackSpeed((float) speed);
                result.success(null);
                break;
            case "setAdaptive":
                vdoPlayer.setSelectedTracks(new Track[]{Track.SET_ADAPTIVE});
                result.success(null);
                break;
            case "setAudioTrack":
                // Fall over; same as video track selection
            case "setVideoTrack":
                int trackId = (int) call.arguments;
                setTrack(trackId);
                result.success(null);
                break;
            case "setSubtitleLanguage":
                String subtitleLang = (String) call.arguments;
                boolean changed = setSubtitleLanguage(subtitleLang);
                result.success(changed);
                break;
            case "getPlaybackProperty":
                String propertyName = (String) call.arguments;
                Object playbackProperty = vdoPlayer.getPlaybackProperty(propertyName);
                result.success(playbackProperty);
                break;
            default:
                result.notImplemented();
        }
    }

    // VdoPlayer.PlaybackEventListener impl

    @Override
    public void onPlayerStateChanged(boolean playWhenReady, int playbackState) {
        if (playerEventSink != null) {
            Map<String, Object> event = new HashMap<>();
            event.put("name", "onPlayerStateChanged");
            event.put("playWhenReady", playWhenReady);
            event.put("playbackState", MessageUtils.playerStateName(playbackState));
            playerEventSink.success(event);
        }
        boolean playing = playbackState != VdoPlayer.STATE_IDLE && playbackState != VdoPlayer.STATE_ENDED && playWhenReady;

        // We are playing the video now. Update the media session state and the PiP
        // window will update the actions.

        if (vdoPlayer != null && !showControls)
            updatePlaybackState(playing ? PlaybackStateCompat.STATE_PLAYING : PlaybackStateCompat.STATE_PAUSED,
                    vdoPlayer.getCurrentTime(), vdoPlayer.getPlaybackSpeed());

        if (!showControls) {
            if (playing) {
                //Request audio focus and notify other players to stop playback.
                if (!requestAudioFocus(audioManager.get())) {
                    Log.i(TAG, "Audio focus not granted");
                }
            } else if (playbackState == VdoPlayer.STATE_ENDED) {
                // Abandon audio focus when playback complete
                audioManager.get().abandonAudioFocus(audioFocusChangeListener);
            }
        }
    }

    @Override
    public void onSeekTo(long millis) {
        methodChannel.invokeMethod("onSeekTo", millis);
    }

    @Override
    public void onProgress(long millis) {
        methodChannel.invokeMethod("onProgress", millis);
    }

    @Override
    public void onBufferUpdate(long bufferTime) {
        methodChannel.invokeMethod("onBufferUpdate", bufferTime);
    }

    @Override
    public void onPlaybackSpeedChanged(float speed) {
        if (playerEventSink != null) {
            Map<String, Object> event = new HashMap<>();
            event.put("name", "onPlaybackSpeedChanged");
            event.put("value", speed);
            playerEventSink.success(event);
        }
    }

    @Override
    public void onLoading(VdoInitParams vdoInitParams) {
        Log.i(TAG, "onLoading");
        methodChannel.invokeMethod("onLoading", VdoInitParamsToMap.convert(vdoInitParams));
    }

    @Override
    public void onLoaded(VdoInitParams vdoInitParams) {
        Log.i(TAG, "onLoaded");
        methodChannel.invokeMethod("onLoaded", VdoInitParamsToMap.convert(vdoInitParams));
        if (loadResult != null && vdoPlayer != null) {
            loadResult.success(MessageUtils.loadResultMap(vdoPlayer.getCurrentMedia()));
            loadResult = null;
        }

        if (playerEventSink != null && vdoPlayer != null) {
            Track[] availableTracks = vdoPlayer.getAvailableTracks();
            Track[] selectedTracks = vdoPlayer.getSelectedTracks();
            Map<String, Object> event = new HashMap<>();
            event.put("name", "onTracksChanged");
            event.put("available", MessageUtils.trackMapList(availableTracks));
            event.put("selected", MessageUtils.trackMapList(selectedTracks));
            playerEventSink.success(event);
        }
    }

    @Override
    public void onLoadError(VdoInitParams vdoInitParams, ErrorDescription errorDescription) {
        Log.e(TAG, "onLoadError " + errorDescription);
        Map<String, Object> arguments = new HashMap<>();
        arguments.put("embedInfo", VdoInitParamsToMap.convert(vdoInitParams));
        arguments.put("errorDescription", MessageUtils.errorInfoMap(errorDescription));
        methodChannel.invokeMethod("onLoadError", arguments);
        if (playerEventSink != null) {
            Map<String, Object> event = new HashMap<>();
            event.put("name", "onError");
            event.put("errorDescription", MessageUtils.errorInfoMap(errorDescription));
            playerEventSink.success(event);
        }
    }

    @Override
    public void onMediaEnded(VdoInitParams vdoInitParams) {
        Log.i(TAG, "onMediaEnded");
        methodChannel.invokeMethod("onMediaEnded", VdoInitParamsToMap.convert(vdoInitParams));
        if (playerEventSink != null) {
            Map<String, Object> event = new HashMap<>();
            event.put("name", "onMediaEnded");
            playerEventSink.success(event);
        }
    }

    @Override
    public void onError(VdoInitParams vdoInitParams, ErrorDescription errorDescription) {
        Map<String, Object> arguments = new HashMap<>();
        arguments.put("embedInfo", VdoInitParamsToMap.convert(vdoInitParams));
        arguments.put("errorDescription", MessageUtils.errorInfoMap(errorDescription));
        methodChannel.invokeMethod("onError", arguments);
        if (playerEventSink != null) {
            Map<String, Object> event = new HashMap<>();
            event.put("name", "onError");
            event.put("errorDescription", MessageUtils.errorInfoMap(errorDescription));
            playerEventSink.success(event);
        }
    }

    @Override
    public void onTracksChanged(Track[] availableTracks, Track[] selectedTracks) {
        Map<String, Object> arguments = new HashMap<>();
        arguments.put("available", MessageUtils.trackMapList(availableTracks));
        arguments.put("selected", MessageUtils.trackMapList(selectedTracks));
        methodChannel.invokeMethod("onTracksChanged", arguments);

        if (playerEventSink != null) {
            Map<String, Object> event = new HashMap<>();
            event.put("name", "onTracksChanged");
            event.put("available", MessageUtils.trackMapList(availableTracks));
            event.put("selected", MessageUtils.trackMapList(selectedTracks));
            playerEventSink.success(event);
        }
    }

    private void setTrack(final int trackId) {
        if (vdoPlayer == null) {
            return;
        }
        // Find track with trackId in available tracks
        Track match = null;
        for (Track track : vdoPlayer.getAvailableTracks()) {
            if (track.id == trackId) {
                match = track;
                break;
            }
        }

        // If track found, select it
        if (match != null) {
            vdoPlayer.setSelectedTracks(new Track[]{match});
        }
    }

    private boolean setSubtitleLanguage(String lang) {
        if (vdoPlayer == null) {
            return false;
        }
        if (lang == null) {
            // Turn subtitles off
            vdoPlayer.setSelectedTracks(new Track[]{Track.DISABLE_CAPTIONS});
            return true;
        } else {
            // Set provided subtitle language if available
            Track[] tracks = vdoPlayer.getAvailableTracks();
            for (Track track : tracks) {
                if (track.type == Track.TYPE_CAPTIONS && lang.equals(track.language)) {
                    vdoPlayer.setSelectedTracks(new Track[]{track});
                    return true;
                }
            }
            return false;
        }
    }

    //Called when activity configuration is changed.
    @Override
    public void accept(Configuration configuration) {
        methodChannel.invokeMethod("onFullscreenAction", configuration.orientation == Configuration.ORIENTATION_LANDSCAPE);
        if (vdoPlayerControlView != null)
            vdoPlayerControlView.setFullscreenState(configuration.orientation == Configuration.ORIENTATION_LANDSCAPE);
    }

    private static class MediaSessionCallback extends MediaSessionCompat.Callback {

        private final VdoPlayer player;

        public MediaSessionCallback(VdoPlayer player) {
            this.player = player;
        }

        @Override
        public void onPlay() {
            super.onPlay();
            //resume playing
            player.setPlayWhenReady(true);
        }

        @Override
        public void onPause() {
            super.onPause();
            //pause playing
            player.setPlayWhenReady(false);
        }

        @Override
        public void onSkipToNext() {
            super.onSkipToNext();
            //Implement this according to your app needs
        }

        @Override
        public void onSkipToPrevious() {
            super.onSkipToPrevious();
            //Implement this according to your app needs
        }
    }

    private boolean requestAudioFocus(AudioManager audioManager) {
        return AudioManager.AUDIOFOCUS_REQUEST_GRANTED == audioManager.requestAudioFocus(audioFocusChangeListener, AudioManager.STREAM_MUSIC, AudioManager.AUDIOFOCUS_GAIN);
    }

    private final AudioManager.OnAudioFocusChangeListener audioFocusChangeListener = new AudioManager.OnAudioFocusChangeListener() {
        @Override
        public void onAudioFocusChange(int focusChange) {
            switch (focusChange) {
                case AudioManager.AUDIOFOCUS_GAIN:
                    //resume playing
                    if (vdoPlayer != null)
                        vdoPlayer.setPlayWhenReady(true);
                    break;
                case AudioManager.AUDIOFOCUS_LOSS_TRANSIENT_CAN_DUCK:
                    //depending on your app reduce audio to a minimum if you want to continue playing or pause
                    break;
                case AudioManager.AUDIOFOCUS_LOSS:
                    //pause playing
                    if (vdoPlayer != null)
                        vdoPlayer.setPlayWhenReady(false);
                case AudioManager.AUDIOFOCUS_LOSS_TRANSIENT:
                default:
                    //stop playing
                    break;
            }
        }
    };
}
