package com.vdocipher.flutter.vdoplayer.offline;

import android.content.Context;
import android.os.Build;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;

import com.vdocipher.aegis.media.MediaInfo;
import com.vdocipher.aegis.media.Track;
import com.vdocipher.aegis.offline.DownloadOptions;
import com.vdocipher.aegis.offline.DownloadRequest;
import com.vdocipher.aegis.offline.DownloadSelections;
import com.vdocipher.aegis.offline.DownloadStatus;
import com.vdocipher.aegis.offline.VdoDownloadManager;
import com.vdocipher.flutter.vdoplayer.MessageUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class VdoDownloader implements VdoDownloadManager.EventListener {
    private VdoDownloadManager vdoDownloadManager;
    private static VdoDownloader vdoDownloader;

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    private VdoDownloader(Context context) {
        vdoDownloadManager = VdoDownloadManager.getInstance(context);
    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public static VdoDownloader getInstance(Context context) {
        if (vdoDownloader == null) {
            vdoDownloader = new VdoDownloader(context);
        }
        return vdoDownloader;
    }

    public void addEventListener(VdoDownloadManager.EventListener listener) {
        vdoDownloadManager.addEventListener(listener);
    }

    public void removeEventListener(VdoDownloadManager.EventListener listener) {
        vdoDownloadManager.removeEventListener(listener);
    }

    public void enqueue(@NonNull DownloadRequest downloadRequest) {
        vdoDownloadManager.enqueueV2(downloadRequest);
    }


    /**
     * Query the VdoDownloadManager about media downloads that have been requested. The results are
     * provided asynchronously to the provided {@link VdoDownloadManager.QueryResultListener}.
     *
     * @param query               a query specifying filters
     * @param queryResultListener a listener for the query results
     */
    public void query(@NonNull final VdoDownloadManager.Query query, @NonNull final VdoDownloadManager.QueryResultListener queryResultListener) {
        vdoDownloadManager.query(query, queryResultListener);
    }

    /**
     * Cancel media downloads and delete them locally.
     *
     * @param mediaIds mediaIds for media to be removed
     */
    public void remove(final String... mediaIds) {
        vdoDownloadManager.remove(mediaIds);
    }

    public void stopDownload(String mediaId) {
        vdoDownloadManager.stopDownload(mediaId);
    }

    public void resumeDownload(String mediaId) {
        vdoDownloadManager.resumeDownload(mediaId);
    }

    /**
     * Stop download for given media ids.
     *
     * @param mediaIds List of media id
     */
    public void stopDownloads(String... mediaIds) {
        vdoDownloadManager.stopDownloads(mediaIds);
    }

    /**
     * Resume download for given media ids.
     *
     * @param mediaIds
     */
    public void resumeDownloads(String... mediaIds) {
        vdoDownloadManager.resumeDownloads(mediaIds);
    }


    @Override
    public void onQueued(String s, DownloadStatus downloadStatus) {

    }

    @Override
    public void onChanged(String s, DownloadStatus downloadStatus) {

    }

    @Override
    public void onCompleted(String s, DownloadStatus downloadStatus) {

    }

    @Override
    public void onFailed(String s, DownloadStatus downloadStatus) {

    }

    @Override
    public void onDeleted(String s) {

    }
}
