import 'package:vdocipher_flutter/src/offline/vdo_downloader.dart';
import 'package:vdocipher_flutter/src/types/media_info.dart';

/// Download status information corresponding to a media download.
class DownloadStatus {
  /// A MediaInfo object containing information about the media this [DownloadStatus] corresponds to.
  final MediaInfo mediaInfo;

  /// Status of this download. One of VdoDownloadManager.STATUS_* constants.
  final int? status;

  /// More detail on status of the download. It's meaning is dependent on the value of field [status].
  final int? reason;

  /// Optional description in addition to [reason]. When not null, this is a json string
  /// which can be parsed to obtain error message, http code, etc.
  final String? reasonDescription;

  /// Overall download progress as percentage.
  final int? downloadPercent;

  /// Path to a poster file for the download; null if poster not available yet.
  final String? poster;

  DownloadStatus(
    this.mediaInfo,
    this.status,
    this.reason,
    this.reasonDescription,
    this.downloadPercent,
    this.poster,
  );

  static DownloadStatus fromMap(Map<Object?, Object?> downloadStatusMap) {
    Map<Object?, Object?> mediaInfoMap =
        downloadStatusMap['mediaInfo'] as Map<Object?, Object?>;
    MediaInfo mediaInfo = MediaInfo(
        mediaId: mediaInfoMap['mediaId'] as String,
        type: MediaInfoType.offline,
        title: mediaInfoMap['title'] as String?,
        description: "",
        duration: Duration(milliseconds: mediaInfoMap['duration'] as int));
    int? status = downloadStatusMap['status']
        as int?; //we only receive this is case of Android
    String? iOSDownloadState = downloadStatusMap['downloadState']
        as String?; //we only receive this is case of iOS
    int? reason = downloadStatusMap['reason'] as int?;
    String? reasonDescription =
        downloadStatusMap['reasonDescription'] as String?;
    int? downloadPercent = downloadStatusMap['percentDownloaded'] as int?;
    String? posterPath = downloadStatusMap['poster'] as String?;

    if (iOSDownloadState != null && iOSDownloadState.isNotEmpty) {
      if (iOSDownloadState == "downloading") {
        status = VdoDownloadManager.STATUS_DOWNLOADING;
      } else if (iOSDownloadState == "not downloaded") {
        status = VdoDownloadManager.STATUS_REMOVING;
      } else if (iOSDownloadState == "downloaded") {
        status = VdoDownloadManager.STATUS_COMPLETED;
      } else if (iOSDownloadState == "paused") {
        status = VdoDownloadManager.STATUS_PAUSED;
      } else {
        status = VdoDownloadManager.STATUS_PENDING;
      }
    }
    return DownloadStatus(mediaInfo, status, reason, reasonDescription,
        downloadPercent, posterPath);
  }
}
