import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:vdocipher_flutter/src/offline/download_options.dart';
import 'package:vdocipher_flutter/src/types/types.dart';

/// Downloads available track options for a media.
class OptionsDownloader {
  late MethodChannel methodChannel;
  Function(DownloadOptions)? onOptionsReceived;
  Function(VdoError)? onOptionsNotReceived;
  late String _otp;
  late String _playbackInfo;
  String? _customPlayerId;

  OptionsDownloader() {
    methodChannel =
        const MethodChannel("com.vdocipher.flutter.option_downloader");
    methodChannel
        .setMethodCallHandler((call) => _handlePlatformMethodCall(call));
  }

  /// Request download options with otp, playbackInfo and custom player id.
  downloadOptionsWithOtp(
      String otp,
      String playbackInfo,
      String? customPlayerId,
      Function(DownloadOptions) onOptionsReceived,
      Function(VdoError)? onOptionsNotReceived) {
    this.onOptionsReceived = onOptionsReceived;
    this.onOptionsNotReceived = onOptionsNotReceived;
    _otp = otp;
    _playbackInfo = playbackInfo;
    _customPlayerId = customPlayerId;
    methodChannel.invokeMethod("downloadOptionsWithOtp", {
      'otp': otp,
      'playbackInfo': playbackInfo,
      'customPlayerId': customPlayerId,
      'videoId': _getVideoId(playbackInfo)
    });
  }

  String? _getVideoId(String playbackInfo) {
    Codec<String, String> stringToBase64 = utf8.fuse(base64);
    Map playbackInfoMap = json.decode(stringToBase64.decode(playbackInfo));
    return playbackInfoMap["videoId"];
  }

  cancel() {
    methodChannel.invokeMethod("cancel");
  }

  Future<dynamic> _handlePlatformMethodCall(MethodCall call) async {
    switch (call.method.toString()) {
      case 'onOptionsReceived':
        Map<Object?, Object?> downloadOptionsMap =
            call.arguments['downloadOptions'];
        DownloadOptions downloadOptions = DownloadOptions.fromMap(
            downloadOptionsMap, _otp, _playbackInfo, _customPlayerId);
        if (onOptionsReceived != null) onOptionsReceived!(downloadOptions);
        break;
      case 'onOptionsNotReceived':
        Map<Object?, Object?> errorDescription =
            call.arguments['errorDescription'];
        VdoError vdoError = VdoError.fromMap(errorDescription);
        if (onOptionsNotReceived != null) onOptionsNotReceived!(vdoError);
        break;
      default:
        throw MissingPluginException();
    }
  }
}
