import 'package:vdocipher_flutter/vdocipher_flutter.dart';

/// Initial parameters specifying the video to load and playback preferences.
class EmbedInfo {
  final bool offline;
  final String? otp;
  final String? playbackInfo;
  final String? mediaId;
  final EmbedInfoOptions? embedInfoOptions;
  final Duration? skipDuration;
  final List<double>? playbackSpeedOptions;

  EmbedInfo(
      {required this.offline,
      this.otp,
      this.playbackInfo,
      this.mediaId,
      this.embedInfoOptions,
      this.skipDuration,
      this.playbackSpeedOptions});

  const EmbedInfo.streaming(
      {required String this.otp,
      required String this.playbackInfo,
      this.embedInfoOptions,
      this.skipDuration,
      this.playbackSpeedOptions = defaultPlaybackSpeedOptions})
      : offline = false,
        mediaId = null;

  const EmbedInfo.offline(
      {required String this.mediaId,
      this.embedInfoOptions,
      this.skipDuration,
      this.playbackSpeedOptions})
      : offline = true,
        otp = null,
        playbackInfo = null;

  Map<String, Object?> toMap() {
    return {
      'offline': offline,
      'otp': otp,
      'playbackInfo': playbackInfo,
      'mediaId': mediaId,
      'embedInfoOptions':
          embedInfoOptions == null ? null : embedInfoOptions!.toMap()
    };
  }

  static EmbedInfo fromMap(Map<Object?, Object?> embedInfo) {
    final bool offline =
        embedInfo["offline"] != null ? embedInfo["offline"] as bool : false;
    final String? otp = embedInfo["otp"] as String?;
    final String? playbackInfo = embedInfo["playbackInfo"] as String?;
    final String? mediaId = embedInfo["mediaId"] as String?;

    final Map<Object?, Object?> embedInfoOptionsMap =
        embedInfo["embedInfoOptions"] as Map<Object?, Object?>;

    final String? safetyNetApiKey =
        embedInfoOptionsMap["safetyNetApiKey"] as String?;
    final String? preferredCaptionsLanguage =
        embedInfoOptionsMap["preferredCaptionsLanguage"] as String?;
    final Duration? startTimeMs = embedInfoOptionsMap["startTimeMs"] != null
        ? Duration(milliseconds: embedInfoOptionsMap["startTimeMs"] as int)
        : null;
    final Duration? endTimeMs = embedInfoOptionsMap["endTimeMs"] != null
        ? Duration(milliseconds: embedInfoOptionsMap["endTimeMs"] as int)
        : null;
    final Duration? resumeTimeMs = embedInfoOptionsMap["resumeTimeMs"] != null
        ? Duration(milliseconds: embedInfoOptionsMap["resumeTimeMs"] as int)
        : null;
    final int? maxVideoBitrateKbps =
        embedInfoOptionsMap["maxVideoBitrateKbps"] != null
            ? embedInfoOptionsMap["maxVideoBitrateKbps"] as int?
            : null;
    final int? bufferingGoalMs = embedInfoOptionsMap["bufferingGoalMs"] != null
        ? embedInfoOptionsMap["bufferingGoalMs"] as int?
        : null;
    final bool disableAnalytics =
        embedInfoOptionsMap["disableAnalytics"] != null
            ? embedInfoOptionsMap["disableAnalytics"] as bool
            : false;
    final bool allowAdbDebugging =
        embedInfoOptionsMap["allowAdbDebugging"] != null
            ? embedInfoOptionsMap["allowAdbDebugging"] as bool
            : false;
    final bool autoplay = embedInfoOptionsMap["autoplay"] != null
        ? embedInfoOptionsMap["autoplay"] as bool
        : false;
    final bool forceLowestBitrate =
        embedInfoOptionsMap["forceLowestBitrate"] != null
            ? embedInfoOptionsMap["forceLowestBitrate"] as bool
            : false;
    final bool forceHighestSupportedBitrate =
        embedInfoOptionsMap["forceHighestSupportedBitrate"] != null
            ? embedInfoOptionsMap["forceHighestSupportedBitrate"] as bool
            : false;
    final String? customPlayerId = embedInfoOptionsMap["customPlayerId"] != null
        ? embedInfoOptionsMap["customPlayerId"] as String?
        : null;
    final List<Object?> techOverride =
        embedInfoOptionsMap["techOverride"] as List<Object?>;
    final Map<Object?, Object?>? configMap =
        embedInfoOptionsMap["configMap"] != null
            ? embedInfoOptionsMap["configMap"] as Map<Object?, Object?>?
            : null;

    EmbedInfoOptions embedInfoOptions = EmbedInfoOptions(
        safetyNetApiKey: safetyNetApiKey,
        preferredCaptionsLanguage: preferredCaptionsLanguage,
        startPosition: startTimeMs,
        endPosition: endTimeMs,
        resumePosition: resumeTimeMs,
        maxVideoBitrateKbps: maxVideoBitrateKbps,
        bufferingGoalMs: bufferingGoalMs,
        disableAnalytics: disableAnalytics,
        allowAdbDebugging: allowAdbDebugging,
        autoplay: autoplay,
        forceLowestBitrate: forceLowestBitrate,
        forceHighestSupportedBitrate: forceHighestSupportedBitrate,
        customPlayerId: customPlayerId,
        techOverride: techOverride,
        configMap: configMap);

    return EmbedInfo(
        offline: offline,
        otp: otp,
        playbackInfo: playbackInfo,
        mediaId: mediaId,
        embedInfoOptions: embedInfoOptions);
  }
}
